@@ -11,6 +11,7 @@ import math
 from scipy import signal
 import matplotlib.pyplot as plt
 from matplotlib import gridspec
+from matplotlib.collections import LineCollection
 
 
 class Pair:
@@ -79,12 +80,9 @@ class Pair:
             raise Exception('Unexpected number of arguments')
                     
         # some sanity checks
-        if self.x.ndim != 1:
-            raise Exception('data must be one dimensional')
-        if self.x.size%2 == 0:
-            raise Exception('data must have odd number of samples')
-        if (self.x.size != self.y.size):
-            raise Exception('x and y must be the same length')
+        if self.x.ndim != 1: raise Exception('data must be one dimensional')
+        if self.x.size%2 == 0: raise Exception('data must have odd number of samples')
+        if (self.x.size != self.y.size): raise Exception('x and y must be the same length')
                    
         # Pair must have a window
         self.set_window(**kwargs)
@@ -96,8 +94,9 @@ class Pair:
         self.cmpvecs = np.eye(2)  
         if ('cmpvecs' in kwargs): self.cmpvecs = kwargs['cmpvecs']
         
-        self.rayvec = [0,0,1] # normal to shear plane, along Z-axis
-        if ('rayvec' in kwargs): self.rayvec = kwargs['rayvec']
+        # self.rayvec = [0,0,1] # normal to shear plane, along Z-axis
+        # if ('rayvec' in kwargs): self.rayvec = kwargs['rayvec']
+        # Always just assume ray vector is normal to components
         
         # source and receiver location info
         if ('srcloc' in kwargs): self.srcloc = kwargs['srcloc']     
@@ -155,16 +154,10 @@ class Pair:
         # find appropriate rotation matrix
         ang = math.radians(degrees)
         # define the new cmpvecs
-        backoff = self.cmpvecs.T
-        self.cmpvecs = np.array([[ math.cos(ang), math.sin(ang)],
-                                 [-math.sin(ang), math.cos(ang)]])
-        # find the rotation matrix. 
-        # Linear algebra: if a and b are rotation matrices, 
-        # then: a.T = inv(a) and b.T = inv(b)
-        # then: dot(a.T,a) = dot(b.T,b) = I
-        # and (multiply by b): dot(dot(b,a.T),a) = b.
-        # i.e., dot(b,a.T) is the rotation matrix that converts a to b.
-        rot = np.dot(self.cmpvecs, backoff)
+        backoff = self.cmpvecs
+        self.cmpvecs = np.array([[ math.cos(ang),-math.sin(ang)],
+                                 [ math.sin(ang), math.cos(ang)]])
+        rot = np.dot(self.cmpvecs.T, backoff)
         # rotate data
         xy = np.dot(rot, self.data())
         self.x, self.y = xy[0], xy[1]
@@ -204,12 +197,13 @@ class Pair:
         # if start/end given
         if len(args) == 2:
             start, end = args 
+            if start > end: raise ValueError('start is larger than end')
             time_centre = (start + end)/2
             time_width = end - start
             tcs = core.time2samps(time_centre,self.delta)
             offset = tcs - self._centresamp()
-            # convert time to nsamples -- must be odd
-            width = core.time2samps(time_width,self.delta,'odd')       
+            # convert time to nsamples -- must be odd (even plus 1 because x units of deltatime needs x+1 samples)
+            width = core.time2samps(time_width,self.delta,'even') + 1     
             self.window = Window(width,offset,**kwargs) 
             return
         else:
@@ -267,8 +261,7 @@ class Pair:
     def snrRH(self):
         data = self.copy()
         data.rotateto(data.pol())
-        data.chop()
-        return core.snrRH(data.data())
+        return core.snrRH(data.chop().data())
 
     def cmpangs(self):
         cmp1 = self.cmpvecs[:,0]
@@ -281,8 +274,8 @@ class Pair:
         Chop data to window
         """
         chop = self.copy()
-        chop.x, chop.y = core.chop(self.x,self.y,window=self.window)
-        self.window.offset = 0
+        chop.x, chop.y = core.chop(chop.x,chop.y,window=chop.window)
+        chop.window.offset = 0
         return chop
         
     def chopt(self):
@@ -310,7 +303,7 @@ class Pair:
         """
         Window width.
         """
-        return self.window.width * self.delta
+        return (self.window.width-1) * self.delta
         
     # Plotting
                 
@@ -344,7 +337,7 @@ class Pair:
         if 'cmplabels' not in kwargs: kwargs['cmplabels'] = self.cmplabels
         ax.plot( t, self.x, label=kwargs['cmplabels'][0])
         ax.plot( t, self.y, label=kwargs['cmplabels'][1])
-        ax.legend()
+        ax.legend(framealpha=0.5)
     
         # set limits
         lim = np.abs(self.data()).max() * 1.1
@@ -368,9 +361,24 @@ class Pair:
     def _ppm(self,ax,**kwargs):
         """Plot particle motion on *ax* matplotlib axis object.
         """
+        
+        data = self.chop()
+        data.rotateto(0)
+        x, y = data.x, data.y
+        t = data.t()
                 
         # plot data
-        ax.plot(self.chop().y,self.chop().x)
+        # ax.plot(self.chop().y,self.chop().x)
+        
+        # multi-colored
+        norm = plt.Normalize(t.min(),t.max())
+        points = np.array([y, x]).T.reshape(-1, 1, 2)
+        segments = np.concatenate([points[:-1], points[1:]], axis=1)
+        lc = LineCollection(segments,cmap='plasma',norm=norm,alpha=0.7)
+        lc.set_array(t)
+        lc.set_linewidth(2)
+        line = ax.add_collection(lc)
+        # plt.colorbar(line)
     
         # set limit
         lim = np.abs(self.data()).max() * 1.1
@@ -380,7 +388,7 @@ class Pair:
         ax.set_ylim(kwargs['lims'])
     
         # set labels
-        if 'cmplabels' not in kwargs: kwargs['cmplabels'] = self.cmplabels
+        if 'cmplabels' not in kwargs: kwargs['cmplabels'] = data.cmplabels
         ax.set_xlabel(kwargs['cmplabels'][1])
         ax.set_ylabel(kwargs['cmplabels'][0])
         
